/* **************************************************************
 * 
 *      Wii - Nunchuck
 *     ================
 *     Uwe Berger; 2016 
 * 
 * Bibliothek zum Abfragen eines Wii-Nunchuck mit einem AVR.
 * 
 * Voraussetzung:
 * --------------
 * Eine I2C-Bibliothek, hier wird die von Peter Fleury verwendet.
 * http://homepage.hispeed.ch/peterfleury/doxygen/avr-gcc-libraries/group__pfleury__ic2master.html
 * 
 * 
 * Inspiration:
 * ------------
 * https://todbot.com/blog/2008/02/18/wiichuck-wii-nunchuck-adapter-available/
 * http://davidegironi.blogspot.de/2012/11/avr-atmega-wii-nunchuck-library-01.html
 * 
 * 
 * ---------
 * Have fun! 
 * 
 * ---------------------------------------------------------------------
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 ***************************************************************
 */

#include "wii_nunchuck.h"

// *******************************************************
void wii_nunchuck_init(void) 
{
	i2c_start_wait(WII_NUNCHUCK_TWIADDR + I2C_WRITE);
	i2c_write(0xF0);
	i2c_write(0x55);
	i2c_stop();
	_delay_ms(1);
	i2c_start_wait(WII_NUNCHUCK_TWIADDR + I2C_WRITE);
	i2c_write(0xFB);
	i2c_write(0x00);
	i2c_stop();
	_delay_ms(1);
}

// *******************************************************
nunchuck_controls_t wii_nunchuck_get_data(void)
{
	uint8_t buf[WII_NUNCHUCK_DATA_LEN]; 
	uint8_t i;
	nunchuck_controls_t ctrl;
	static uint8_t old_button_c =0;
	static uint8_t old_button_z =0;
	static uint8_t old_button_xl=0;
	static uint8_t old_button_xr=0;
	static uint8_t old_button_yu=0;
	static uint8_t old_button_yd=0;

	// Datenuebertragung initiieren
	i2c_start_wait(WII_NUNCHUCK_TWIADDR + I2C_WRITE);
	i2c_write(0x00);
	i2c_stop();
	_delay_ms(1);
	// 6 Byte sollten ankommen...
	i2c_start_wait(WII_NUNCHUCK_TWIADDR + I2C_READ);
	for(i=0; i < WII_NUNCHUCK_DATA_LEN; i++) {
		if(i == WII_NUNCHUCK_DATA_LEN-1)
			buf[i] = i2c_readNak();
		else
			buf[i] = i2c_readAck();
	}
	i2c_stop();
	// empfangene Werte in Struktur uebernehmen
	ctrl.joy_x = buf[0];
	ctrl.joy_y = buf[1];
	// ...Button-Bits sind low-aktiv!
	ctrl.button_c = !((buf[5] >> 1) & 0b00000001);
	ctrl.button_z = !(buf[5]        & 0b00000001);
	// ...deren Repeat-Werte setzen
	ctrl.button_c_repeat = (ctrl.button_c && old_button_c);
	old_button_c = ctrl.button_c;
	ctrl.button_z_repeat = (ctrl.button_z && old_button_z);
	old_button_z = ctrl.button_z;
	// ..."Joystick-Button"
	ctrl.button_xl = (buf[0] < BUTTON_XL_LIMIT);
	ctrl.button_xr = (buf[0] > BUTTON_XR_LIMIT);
	ctrl.button_yu = (buf[1] > BUTTON_YU_LIMIT);
	ctrl.button_yd = (buf[1] < BUTTON_YD_LIMIT);
	// ...deren Repeat-Werte setzen
	ctrl.button_xl_repeat = (ctrl.button_xl && old_button_xl);
	old_button_xl = ctrl.button_xl;
	ctrl.button_xr_repeat = (ctrl.button_xr && old_button_xr);
	old_button_xr = ctrl.button_xr;
	ctrl.button_yu_repeat = (ctrl.button_yu && old_button_yu);
	old_button_yu = ctrl.button_yu;
	ctrl.button_yd_repeat = (ctrl.button_yd && old_button_yd);
	old_button_yd = ctrl.button_yd;
	// ...10-Bit-Beschleunigungswerte auf 2 Byte verteilt
	ctrl.accel_x = (buf[2] << 2) + ((buf[5] >> 2) & 0b00000011);
	ctrl.accel_y = (buf[3] << 2) + ((buf[5] >> 4) & 0b00000011);
	ctrl.accel_z = (buf[4] << 2) + ((buf[5] >> 6) & 0b00000011);
	return ctrl;
}
